import urllib

from django import http
from django.utils import simplejson

from model import AutocompleteModel


class AutocompleteRestModel(AutocompleteModel):
    widget_js_attributes = {'bootstrap': 'rest_model'}

    @property
    def model(self):
        return self.choices.model

    def post(self, request, *args, **kwargs):
        value = request.POST['value']
        pk = self.download_choice(value)
        return http.HttpResponse(pk, status=201)

    def get_source_url(self, limit):
        """
        Return an API url for the current autocomplete request.

        By default, return self.source_url with the data dict returned by
        get_source_url_data().
        """
        return '%s?%s' % (self.source_url, urllib.urlencode(
            self.get_source_url_data(limit)))

    def get_source_url_data(self, limit):
        """
        Given a limit of items, return a dict of data to send to the API.

        By default, it passes current request GET arguments, along with format:
        'json' and the limit.
        """
        data = {}
        if self.request:
            for key, value in self.request.GET.items():
                data[key] = value

        data.update({
            'format': 'json',
            'limit': limit,
        })

        return data

    def model_for_source_url(self, url):
        """
        Take an URL from the API this remote channel is supposed to work with,
        return the model class to use for that url.

        It is only needed for the default implementation of download(), because
        it has to follow relations recursively.

        By default, it will return the model of self.choices.
        """
        return self.choices.model

    def choices_for_request(self):
        choices = super(AutocompleteRestModel, self).choices_for_request()
        unicodes = [unicode(choice) for choice in choices]

        slots = self.limit_choices - len(choices)

        if slots > 0:
            choices = list(choices)

            for choice in self.get_remote_choices(slots):
                # avoid data that's already in local
                if unicode(choice) in unicodes:
                    continue

                choices.append(choice)

        return choices

    def get_remote_choices(self, max):
        """
        Parses JSON from the API, return model instances.

        The JSON should contain a list of dicts. Each dict should contain the
        attributes of an object. Relation attributes should be represented by
        their url in the API, which is set to model._source_url.
        """
        url = self.get_source_url(max)

        try:
            fh = urllib.urlopen(url)
            body = fh.read()
        except:
            return
        else:
            for data in simplejson.loads(body):
                url = data.pop('url')

                for name in data.keys():
                    field = self.model._meta.get_field_by_name(name)[0]
                    if getattr(field, 'rel', None):
                        data.pop(name)
                model = self.model(**data)
                model.pk = url
                yield model

    def download_choice(self, choice):
        """
        Take a choice's dict representation, return it's local pk which might
        have been just created.

        If your channel works with 0 to 1 API call, consider overriding this
        method.
        If your channel is susceptible of using several different API calls,
        consider overriding download().
        """
        return self.download(choice).pk

    def download(self, url):
        """
        Given an url to a remote object, return the corresponding model from
        the local database.

        The default implementation expects url to respond with a JSON dict of
        the attributes of an object.

        For relation attributes, it expect the value to be another url that
        will respond with a JSON dict of the attributes of the related object.

        It calls model_for_source_url() to find which model class corresponds
        to which url. This allows download() to be recursive.
        """
        model_class = self.model_for_source_url(url)

        fh = urllib.urlopen(url)
        data = simplejson.loads(fh.read())
        data.pop('url')
        fh.close()

        uniques = [f.name for f in model_class._meta.fields if f.unique]
        unique_data = {}

        for key, value in data.items():
            if key not in uniques:
                continue

            field = model_class._meta.get_field_by_name(key)[0]
            if getattr(field, 'rel', False):
                continue

            unique_data[key] = value

        if not unique_data:
            assert self.get_or_create_by, 'get_or_create_by needed'

            for key in self.get_or_create_by:
                if key in data.keys() and data[key]:
                    unique_data[key] = data[key]

            if not len(unique_data.keys()):
                raise Exception('cannot check if this model exists locally')

        try:
            model = model_class.objects.get(**unique_data)
        except model_class.DoesNotExist:
            model = model_class(**unique_data)

        for key, value in data.items():
            is_string = isinstance(value, (unicode, str))
            field = model_class._meta.get_field_by_name(key)[0]

            if getattr(field, 'rel', None) and is_string:
                setattr(model, key, self.download(value))
            else:
                setattr(model, key, value)

        model.save()

        return model



from django.db.models import Q

__all__ = ('AutocompleteModel', )


class AutocompleteModel(object):
    limit_choices = 20
    choices = None
    search_fields = None

    def choice_value(self, choice):
        return choice.pk

    def choice_label(self, choice):
        return unicode(choice)

    def order_choices(self, choices):
        order_by = getattr(self, 'order_by', None)
        if order_by:
            return choices.order_by(order_by)
        return choices

    def choices_for_values(self):
        assert self.choices is not None, 'choices should be a queryset'
        return self.order_choices(self.choices.filter(pk__in=self.values or [])
            )[0:self.limit_choices]

    def choices_for_request(self):
        assert self.choices is not None, 'choices should be a queryset'
        assert self.search_fields, 'autocomplete.search_fields must be set'
        q = self.request.GET.get('q', '')

        conditions = Q()
        if q:
            for search_field in self.search_fields:
                conditions |= Q(**{search_field + '__icontains': q})

        return self.order_choices(self.choices.filter(
            conditions))[0:self.limit_choices]

    def validate_values(self):
        return len(self.choices_for_values()) == len(self.values)



import autocomplete_light
autocomplete_light.autodiscover()

from registry import RegistryTestCase
from templatetags import DataAttributesTestCase
from generic import GenericModelFormTestCase
from generic_m2m import AutocompleteGenericM2MTestCase
from widget import WidgetTestCase

from autocomplete.choice_list import AutocompleteChoiceListTestCase
from autocomplete.list import AutocompleteListTestCase
from autocomplete.model import AutocompleteModelTestCase
from autocomplete.template import AutocompleteModelTemplateTestCase
from autocomplete.generic import AutocompleteGenericTestCase



import unittest

from autocomplete_light.templatetags.autocomplete_light_tags import *


class DataAttributesTestCase(unittest.TestCase):
    def test_without_prefix(self):
        tests = (
            {
                'fixture': {
                    'foo': 'bar',
                },
                'expected': 'data-foo="bar"',
            },
            {
                'fixture': {
                    'foo': 'bar',
                    'test_underscore': 'example',
                },
                'expected': 'data-foo="bar" data-test-underscore="example"',
            },
            {
                'fixture': {
                    'foo': 'bar',
                },
                'prefix': 'autocomplete-',
                'expected': 'data-autocomplete-foo="bar"',
            },
        )

        for test in tests:
            result = autocomplete_light_data_attributes(test['fixture'],
                test.get('prefix', ''))
            self.assertEqual(result, test['expected'],
                u'Got %s for %s' % (result, test))



from .case import *

from django.template import Template, Context

from django.contrib.auth.models import User


class AutocompleteMock(autocomplete_light.AutocompleteModelTemplate):
    limit_choices = 2
    choices = User.objects.all()
    search_fields = ('username', 'email')

    choice_template = u'<li data-value="{{ choice.pk }}">{{ choice }}</li>'
    autocomplete_template = u''.join([
        u'{% load autocomplete_light_tags %}',
        u'<ul>',
        u'{% for choice in choices %}',
        u'{{ choice|autocomplete_light_choice_html:autocomplete }}',
        u'{% endfor %}',
        u'</ul>',
    ])

    def render_template_context(self, template, extra_context=None):
        context = self.get_base_context()
        context.update(extra_context or {})

        template = Template(template)
        return template.render(Context(context))


class AutocompleteModelTemplateTestCase(AutocompleteTestCase):
    autocomplete_mock = AutocompleteMock

    def setUp(self):
        User.objects.all().delete()
        self.abe = User(username='Abe', email='sales@example.com')
        self.jack = User(username='Jack', email='jack@example.com')
        self.james = User(username='James', email='sales@example.com')
        self.john = User(username='John', email='sales@example.com')

        self.abe.save()
        self.jack.save()
        self.james.save()
        self.john.save()

    def get_choices_for_values_tests(self):
        return []

    def get_choices_for_request_tests(self):
        return []

    def get_validate_tests(self):
        return []

    def get_autocomplete_html_tests(self):
        return (
            {
                'fixture': make_get_request('q=j'),
                'expected': u''.join([
                    u'<ul>',
                    u'<li data-value="%s">%s</li>' % (
                        self.jack.pk, unicode(self.jack)),
                    u'<li data-value="%s">%s</li>' % (
                        self.james.pk, unicode(self.james)),
                    u'</ul>',
                ])
            },
            {
                'fixture': make_get_request(),
                'expected': u''.join([
                    u'<ul>',
                    u'<li data-value="%s">%s</li>' % (
                        self.abe.pk, unicode(self.abe)),
                    u'<li data-value="%s">%s</li>' % (
                        self.jack.pk, unicode(self.jack)),
                    u'</ul>',
                ])
            },
        )

    def get_widget_tests(self):
        return []



"""
The registry module provides tools to maintain a registry of autocompletes.

The first thing that should happen when django starts is registration of
autocompletes. It should happen first, because autocompletes are required for
widgets. And autocomplete widgets are required for forms. And forms are
required for ModelAdmin.

It looks like this:

- in ``yourapp/autocomplete_light_registry.py``, register your autocompletes
  with ``autocomplete_light.register()``,
- in ``urls.py``, do ``autocomplete_light.autodiscover()`` **before**
  ``admin.autodiscover()``.

AutocompleteRegistry
    Subclass of Python's dict type with registration/unregistration methods.

registry
    Instance of AutocompleteRegistry.

register
    Proxy registry.register.

autodiscover
    Find autocompletes and fill registry.
"""

from django.db import models

from .autocomplete import AutocompleteModelBase

__all__ = ('AutocompleteRegistry', 'registry', 'register', 'autodiscover')


class AutocompleteRegistry(dict):
    """
    Dict with some shortcuts to handle a registry of autocompletes.
    """

    def __init__(self):
        self._models = {}

    def autocomplete_for_model(self, model):
        """ Return the autocomplete class for a given model. """
        try:
            return self._models[model]
        except KeyError:
            return

    def unregister(self, name):
        """ Unregister a autocomplete. """
        autocomplete = self[name]
        del self[name]

        try:
            if autocomplete.choices.model:
                del self._models[autocomplete.choices.model]
        except AttributeError:
            pass

    def register(self, *args, **kwargs):
        """
        Register an autocomplete.

        Two unordered arguments are accepted, at least one should be passed:

        - a model if not a generic autocomplete,
        - an autocomplete class if necessary, else one will be generated.

        'name' is also an acceptable keyword argument, that can be used to
        override the default autocomplete name (which is its class name).

        In addition, keyword arguments will be set as class attributes. For
        thread safety reasons, a copy of the autocomplete class is stored in
        the registry.
        """
        autocomplete = None
        model = None

        assert len(args) <= 2, 'register takes at most 2 args'
        assert len(args) > 0, 'register takes at least 1 arg'

        for arg in args:
            if issubclass(arg, models.Model):
                model = arg
            else:
                autocomplete = arg

        if not model:
            try:
                model = autocomplete.choices.model
            except AttributeError:
                pass

        if model:
            self._register_model_autocomplete(model, autocomplete, **kwargs)
        else:
            name = kwargs.get('name', autocomplete.__name__)
            autocomplete = type(name, (autocomplete,), kwargs)
            self._register_autocomplete(autocomplete)

    def _register_model_autocomplete(self, model, autocomplete=None,
                                    name=None, **kwargs):

        if name is not None:
            pass
        elif autocomplete is not None:
            if autocomplete.__name__.find(model.__name__) == 0:
                name = autocomplete.__name__
            else:
                name = '%s%s' % (model.__name__, autocomplete.__name__)
        else:
            name = '%sAutocomplete' % model.__name__

        if autocomplete is None:
            base = AutocompleteModelBase
        else:
            base = autocomplete

        if base.choices is None:
            kwargs['choices'] = model.objects.all()

        if base.search_fields is None and 'search_fields' not in kwargs:
            try:
                model._meta.get_field('name')
            except:
                raise Exception(u'Add search_fields kwargs to .register(%s)'
                    % model.__name__)
            else:
                kwargs['search_fields'] = ['name']

        autocomplete = type(name, (base,), kwargs)

        self._register_autocomplete(autocomplete)
        self._models[model] = autocomplete

    def _register_autocomplete(self, autocomplete):
        """
        Register a autocomplete without model, like a generic autocomplete.
        """
        self[autocomplete.__name__] = autocomplete


def _autodiscover(registry):
    """See documentation for autodiscover (without the underscore)"""
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's admin module.
        try:
            before_import_registry = copy.copy(registry)
            import_module('%s.autocomplete_light_registry' % app)
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            # (see #8245).
            registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an admin module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'autocomplete_light_registry'):
                raise

registry = AutocompleteRegistry()


def autodiscover():
    """
    Check all apps in INSTALLED_APPS for stuff related to autocomplete_light.

    For each app, autodiscover imports app.autocomplete_light_registry if
    available, resulting in execution of register() statements in that module,
    filling registry.

    Consider a standard app called 'cities_light' with such a structure::

        cities_light/
            __init__.py
            models.py
            urls.py
            views.py
            autocomplete_light_registry.py

    With such a autocomplete_light_registry.py::

        from models import City, Country
        import autocomplete_light
        autocomplete_light.register(City)
        autocomplete_light.register(Country)

    When autodiscover() imports cities_light.autocomplete_light_registry, both
    CityAutocomplete and CountryAutocomplete will be registered. For details on
    how these autocomplete classes are generated, read the documentation of
    AutocompleteRegistry.register.
    """
    _autodiscover(registry)


def register(*args, **kwargs):
    """Proxy registry.register"""
    return registry.register(*args, **kwargs)



import unittest

from django.contrib.auth.models import User
from django.db import models

import autocomplete_light


class Noname(models.Model):
    number = models.CharField(max_length=100)


class Foo(models.Model):
    name = models.CharField(max_length=100)


class Bar(autocomplete_light.AutocompleteModelBase):
    pass


class Generic(autocomplete_light.AutocompleteGenericBase):
    choices = (
        User.objects.all(),
    )
    search_fields = (
        ('username',),
    )


class RegistryTestCase(unittest.TestCase):
    def setUp(self):
        self.registry = autocomplete_light.AutocompleteRegistry()

    def test_register_model(self):
        self.registry.register(Foo)
        self.assertIn('FooAutocomplete', self.registry.keys())

    def test_register_model_and_autocomplete(self):
        self.registry.register(Foo, Bar)
        self.assertIn('FooBar', self.registry.keys())

    def test_register_autocomplete(self):
        self.registry.register(Bar)
        self.assertIn('Bar', self.registry.keys())

    def test_unregister(self):
        self.registry.register(Bar)
        self.registry.unregister('Bar')
        self.assertEqual(self.registry.keys(), [])

    def test_register_with_kwargs(self):
        self.registry.register(Foo, search_name='search_name')
        self.assertEqual(self.registry['FooAutocomplete'].search_name,
            'search_name')

    def test_register_with_autocomplete_and_kwargs(self):
        self.registry.register(Foo, Bar, search_name='search_name')
        self.assertEqual(self.registry['FooBar'].search_name,
            'search_name')

    def test_register_with_custom_name(self):
        self.registry.register(Foo, Bar, name='BarFoo')
        self.assertIn('BarFoo', self.registry.keys())
        self.assertEqual(self.registry['BarFoo'].__name__, 'BarFoo')

    def test_register_no_name_fail(self):
        try:
            self.registry.register(Noname)
            self.fail('Should raise an exception when registering noname')
        except:
            pass

    def test_register_no_name_pass(self):
        self.registry.register(Noname, search_fields=('number',))

    def test_register_generic_with_custom_name(self):
        self.registry.register(Generic, name='foo')
        self.assertTrue('foo' in self.registry.keys())



from .case import *

from django.contrib.auth.models import User


class AutocompleteModelMock(autocomplete_light.AutocompleteModelBase):
    limit_choices = 2
    choices = User.objects.all()
    search_fields = ('username', 'email')


class FormMock(forms.Form):
    x = forms.ModelChoiceField(queryset=AutocompleteModelMock.choices,
        widget=autocomplete_light.ChoiceWidget(
            autocomplete=AutocompleteModelMock))


class MultipleFormMock(forms.Form):
    x = forms.ModelMultipleChoiceField(queryset=AutocompleteModelMock.choices,
        widget=autocomplete_light.MultipleChoiceWidget(
            autocomplete=AutocompleteModelMock))


class AutocompleteModelTestCase(AutocompleteTestCase):
    autocomplete_mock = AutocompleteModelMock

    def setUp(self):
        User.objects.all().delete()
        self.abe = User(username='Abe', email='sales@example.com')
        self.jack = User(username='Jack', email='jack@example.com')
        self.james = User(username='James', email='sales@example.com')
        self.john = User(username='John', email='sales@example.com')

        self.abe.save()
        self.jack.save()
        self.james.save()
        self.john.save()

    def assert_choices_equal(self, result, test):
        self.assertEqual(list(result), test['expected'])

    def get_choices_for_values_tests(self):
        return (
            {
                'fixture': [1, 4],
                'expected': [
                    self.abe,
                    self.john,
                ]
            },
        )

    def get_choices_for_request_tests(self):
        return (
            {
                'fixture': make_get_request('q=j'),
                'expected': [
                    self.jack,
                    self.james,
                ]
            },
            {
                'fixture': make_get_request('q=sale'),
                'expected': [
                    self.abe,
                    self.james,
                ]
            },
            {
                'fixture': make_get_request(),
                'expected': [
                    self.abe,
                    self.jack,
                ]
            }
        )

    def get_validate_tests(self):
        return (
            {
                'fixture': [1, 4],
                'expected': True,
            },
            {
                'fixture': [1, 4, 123],
                'expected': False,
            },
        )

    def get_autocomplete_html_tests(self):
        return (
            {
                'fixture': make_get_request('q=j'),
                'expected': u''.join([
                    '<div data-value="%s">%s</div>' % (
                        self.jack.pk, unicode(self.jack)),
                    '<div data-value="%s">%s</div>' % (
                        self.james.pk, unicode(self.james)),
                ])
            },
            {
                'fixture': make_get_request(),
                'expected': u''.join([
                    '<div data-value="%s">%s</div>' % (
                        self.abe.pk, unicode(self.abe)),
                    '<div data-value="%s">%s</div>' % (
                        self.jack.pk, unicode(self.jack)),
                ])
            },
        )

    def get_widget_tests(self):
        return (
            {
                'form_class': FormMock,
                'fixture': 'x=4',
                'expected_valid': True,
                'expected_data': self.john,
            },
            {
                'form_class': FormMock,
                'fixture': 'x=3&x=4',
                'expected_valid': True,
                'expected_data': self.john,
            },
            {
                'fixture': 'x=abc',
                'expected_valid': False,
            },
            {
                'form_class': MultipleFormMock,
                'fixture': 'x=4&x=2',
                'expected_valid': True,
                'expected_data': [self.jack, self.john],
            },
            {
                'fixture': 'x=abc&x=2',
                'expected_valid': False,
            },
        )



import types

from django.template import loader

from .base import AutocompleteBase


class AutocompleteTemplate(AutocompleteBase):
    """
    This replacement for AutocompleteBase supports two new attributes:

    choice_template
        Name of the template to use to render a choice in the autocomplete. If
        none is specified, then ``AutocompleteBase`` will render the choice.

    autocomplete_template
        Name of the template to use to render the autocomplete. Again, fall
        back on ``AutocompleteBase`` if this is None.
    """
    choice_template = None
    autocomplete_template = None

    def get_base_context(self):
        """
        Return a dict to use as base context for all templates.

        It contains:

        - ``{{ request }}`` if available,
        - ``{{ autocomplete }}`` the "self" instance.
        """
        return {
            'request': self.request,
            'autocomplete': self,
        }

    def render_template_context(self, template, extra_context=None):
        """
        Render ``template`` with base context and ``extra_context``.
        """
        context = self.get_base_context()
        context.update(extra_context or {})
        return loader.render_to_string(template, context)

    def autocomplete_html(self):
        """
        Render ``autocomplete_template`` with base context and ``{{ choices
        }}``. If ``autocomplete_template`` is none then fall back on
        ``AutocompleteBase``.
        """
        if self.autocomplete_template:
            choices = self.choices_for_request()

            return self.render_template_context(self.autocomplete_template,
                {'choices': choices})
        else:
            return super(AutocompleteTemplate, self).autocomplete_html()

    def choice_html(self, choice):
        """
        Render choice_template with base context and ``{{ choice }}``. If
        ``choice_template`` is none then fall back on ``AutocompleteBase``.
        """
        if self.choice_template:
            return self.render_template_context(self.choice_template,
                {'choice': choice})
        else:
            return super(AutocompleteTemplate, self).choice_html(choice)



import time

from django.test import LiveServerTestCase
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.firefox.webdriver import WebDriver

class WidgetTestCase(LiveServerTestCase):
    fixtures = ['cities_light.json']

    @classmethod
    def setUpClass(cls):
        cls.selenium = WebDriver()
        super(WidgetTestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(WidgetTestCase, cls).tearDownClass()
        cls.selenium.quit()

    def autocomplete_visible(self, id=None):
        if id is None: id = self.default_id
        return self.autocomplete_element(id=None).is_displayed()

    def autocomplete_element(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_element_by_css_selector(
            '.yourlabs-autocomplete.outer-container.id-%s' % id)

    def autocomplete_hilighted_choice_element(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_element_by_css_selector(
            '.yourlabs-autocomplete.inner-container.id-%s [data-value].hilight' % id)


    def autocomplete_choice_elements(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_elements_by_css_selector(
            '.yourlabs-autocomplete.inner-container.id-%s [data-value]' % id)

    def input_element(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_element_by_css_selector('#' + id)

    def input_visible(self, id=None):
        if id is None: id = self.default_id
        return self.input_element(id=None).is_displayed()

    def widget_choice_elements(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_elements_by_css_selector(
            '#%s .deck [data-value]' % id.replace('_text', '-wrapper'))

    def widget_select_element(self, id=None):
        return self.widget_element(id).find_element_by_tag_name('select')

    def widget_element(self, id=None):
        if id is None: id = self.default_id
        return self.selenium.find_element_by_css_selector(
            '#%s' % id.replace('_text', '-wrapper'))

    def save(self):
        return self.selenium.find_element_by_css_selector(
            'input[name=_continue]').click()

    def test_login(self):
        self.selenium.get('%s%s' % (self.live_server_url, '/admin/'))
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys('test')
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys('test')
        self.selenium.find_element_by_xpath('//input[@value="Log in"]').click()
        self.selenium.get('%s%s' % (self.live_server_url, '/admin/fk_autocomplete/address/add'))

        self.default_id = 'id_city_text'
        time.sleep(1)

        self.input_element().send_keys('par')
        time.sleep(1)

        self.assertTrue(self.autocomplete_visible())
        self.assertEqual(20, len(self.autocomplete_choice_elements()))


        # select paris
        self.autocomplete_choice_elements()[1].click()

        time.sleep(1)
        self.assertFalse(self.autocomplete_visible())
        self.assertFalse(self.input_visible())

        selected = self.widget_choice_elements()
        self.assertEqual(1, len(selected))
        self.assertTrue('Paris' in selected[0].text)
        self.assertTrue('France' in selected[0].text)


        self.save()
        time.sleep(1)


        self.assertFalse(self.autocomplete_visible())
        self.assertFalse(self.input_visible())

        # remove
        selected = self.widget_choice_elements()
        selected[0].find_element_by_css_selector('.remove').click()

        self.assertFalse(self.autocomplete_visible())
        self.assertTrue(self.input_visible())

        self.input_element().send_keys('par')
        time.sleep(1)

        self.keyboard_test()

        self.assertEqual('',
            self.input_element().get_attribute('value'))

    def keyboard_test(self):
        tests = (
            {
                'key': Keys.ARROW_DOWN,
                'expected': 0,
            },
            {
                'key': Keys.ARROW_DOWN,
                'expected': 1,
            },
            {
                'key': Keys.ARROW_UP,
                'expected': 0,
            },
            {
                'key': Keys.ARROW_UP,
                'expected': -1,
            },
            {
                'key': Keys.ARROW_UP,
                'expected': -2,
            },
        )

        for test in tests:
            self.input_element().send_keys(test['key'])

            self.assertEqual(
                self.autocomplete_choice_elements()[test['expected']].id,
                self.autocomplete_hilighted_choice_element().id
            )

        self.input_element().send_keys(Keys.ENTER)
        self.assertEqual(
            self.autocomplete_choice_elements()[test['expected']].get_attribute('data-value'),
            self.widget_select_element().get_attribute('value'),
        )
